%{
    // Parser assignment template grammar file for Bison
#define __GRAM_TAB_H__
#include "slideparser.h"
%}

%{
        // Semantic Stack Definition
%}
%union {
  CHAR c;
  INT_32 i;
  UINT_32 ui;
  FLOAT f;
  CHAR *str;
  CListInt *pCListInt;
  DynInt *dyni;
  DynFloat *dynf;
  DynVector *dynv;
  DynPoint *dynpt;
}

%{
#define yylloc slide_lloc
#define yyerror slide_error
void slide_error(char* s);

#define yylex slide_lex
extern int slide_lex();

extern YYLTYPE slide_lloc;
extern int yyparse(void);
extern CScanner *slide_gpCScanner;

extern int slide_leng;
extern VOID slide_BeginInitial();
extern YY_BUFFER_STATE slide_GetCurrentBuffer();
extern YY_BUFFER_STATE slide_CreateBuffer(FILE *fpFile);
extern VOID slide_DeleteBuffer(YY_BUFFER_STATE yyBufferState);
extern VOID slide_SwitchToBuffer(YY_BUFFER_STATE yyBufferState);
extern VOID slide_SetValueInt(int i);
extern VOID slide_SetValueFloat(float f);
extern VOID slide_SetValueString(CHAR *pc);

CSLIDEParser *slide_gpCSLIDEParser = NULL;

%}

%{
        // Reserved Word Definitions
%}

%token rwTCL_INIT rwTCL_UPDATE rwTCL_TAG
%token rwWINDOW rwEND_WINDOW rwVIEWPORT rwEND_VIEWPORT
%token rwORIGIN rwSIZE rwBACKGROUND
%token rwCAMERA rwEND_CAMERA
%token rwPROJECTION rwFRUSTUM
%token rwLIGHT rwEND_LIGHT rwTYPE rwDEAD_DISTANCE rwFALL_OFF rwANGULAR_FALL_OFF
%token rwSLF_AMBIENT rwSLF_DIRECTIONAL rwSLF_POINT rwSLF_SPOT
%token rwRENDER rwEND_RENDER
%token rwGROUP rwEND_GROUP rwINSTANCE rwEND_INSTANCE
%token rwTRANSLATE rwSCALE rwROTATE rwLOOK_AT rwEND_LOOK_AT rwEYE rwTARGET rwUP
%{
//%token rwGENERAL
%}
%token rwOBJECT rwEND_OBJECT rwFACE rwEND_FACE rwPOINT rwEND_POINT
%token rwSURFACE rwEND_SURFACE 
%token rwCOLOR rwREFLECTIVITY rwEXPONENT rwMETALLIC
%token rwSLF_INHERIT rwID
%token rwSLF_OFF rwSLF_ON
%token rwSOLID rwSLF_SOLID rwSLF_HOLLOW 
%token rwSHADING rwSLF_WIRE rwSLF_FLAT rwSLF_GOURAUD rwSLF_PHONG
%token rwLOD rwSLF_BOUND rwSLF_EDGES rwSLF_FULL
%token rwSLF_PARALLEL rwSLF_PERSPECTIVE
%token rwNORMAL rwTEXTURE

%{
        // Operator Definitions
%}

%token '(' ')'

%{
        // Constants and Identifier Definitions
%}

%token <c> tokCONST_CHAR
%token <i> tokCONST_INT
%token <f> tokCONST_FLOAT
%token <str> tokCONST_STRING
%token <str> tokIDENTIFIER
%token <str> tokTCL_BLOCK


%type <dyni> dyn_onoff_flag
%type <i> onoff_flag
%type <dyni> dyn_solid_flag
%type <i> solid_flag
%type <dyni> dyn_shading_flag
%type <i> shading_flag
%type <dyni> dyn_lod_flag
%type <i> lod_flag
%type <dyni> dyn_projection_flag
%type <i> projection_flag
%type <dyni> dyn_lighttype_flag
%type <i> lighttype_flag
%type <str> surface_id
%type <pCListInt> id_list
%type <pCListInt> path
%type <f> value
%type <dynf> dynvalue
%type <dynv> dynvector
%type <dynv> dynvector2d
%type <dynpt> dynpoint

%%

world
	: geometry_list
		{ 
		}

geometry_list
	:
		{
		}
	| geometry_list rwTCL_INIT tokTCL_BLOCK
		{
		}
	| geometry_list rwTCL_UPDATE tokTCL_BLOCK
		{
		}
	| geometry_list render
		{
		}
	| geometry_list window
		{
		}
	| geometry_list viewport
		{
		}
	| geometry_list camera
		{
		}
	| geometry_list light
		{
		}
	| geometry_list group
		{
		}
	| geometry_list instance
		{
		}
	| geometry_list object
		{
		}
	| geometry_list face
		{
		}
	| geometry_list point
		{
		}
	| geometry_list surface
		{
		}
	;

window
	: window_start window_middle rwEND_WINDOW
		{
		  slide_gpCSLIDEParser->EndWindow();
		}
	;

window_start
	: rwWINDOW tokIDENTIFIER
		{
		  slide_gpCSLIDEParser->AddWindow(@1.text, @1.first_line, 
						   @1.first_column, $2);
		}
	;

window_middle
	: 
		{
		}
	| window_middle rwORIGIN '(' value value ')'
		{
		  slide_gpCSLIDEParser->SetWindowOrigin($4, $5);
		}
	| window_middle rwSIZE '(' value value ')'
		{
		  slide_gpCSLIDEParser->SetWindowSize($4, $5);
		}
	| window_middle rwBACKGROUND dynvector
		{
		  slide_gpCSLIDEParser->SetWindowBackground($3);
		}
	;

viewport
	: viewport_start viewport_middle rwEND_VIEWPORT
		{
		}
	;

viewport_start
	: rwVIEWPORT tokIDENTIFIER tokIDENTIFIER
		{
		}
	;

viewport_middle
	: 
		{
		}
	| viewport_middle rwORIGIN dynvector2d
		{
		}
	| viewport_middle rwSIZE dynvector2d
		{
		}
	;

camera
	: camera_start camera_middle rwEND_CAMERA
		{
		}
	;

camera_start
	: rwCAMERA tokIDENTIFIER
		{
		}
	;

camera_middle
	: 
		{
		}
	| camera_middle rwPROJECTION dyn_projection_flag
		{
		}
	| camera_middle rwFRUSTUM dynvector dynvector
		{
		}
	;

light
	: light_start light_middle rwEND_LIGHT
		{
		}
	;

light_start
	: rwLIGHT tokIDENTIFIER
		{
		}
	;

light_middle
	: 
		{
		}
	| light_middle rwTYPE dyn_lighttype_flag
		{
		}	
	| light_middle rwCOLOR dynvector
		{
		}
	| light_middle rwDEAD_DISTANCE dynvalue
		{
		}
	| light_middle rwFALL_OFF dynvalue
		{
		}
	| light_middle rwANGULAR_FALL_OFF dynvalue
		{
		}
	;

render
	: render_start render_middle rwEND_RENDER
		{
		}
	;

render_start
	: rwRENDER tokIDENTIFIER path tokIDENTIFIER
		{
		}
	;

render_middle
	: 
		{
		}
	| render_middle rwTCL_TAG tokIDENTIFIER
		{
		}
	| render_middle rwLOD dyn_lod_flag
		{
		}
	| render_middle rwLIGHT path
		{
		}
	| render_middle rwLIGHT path dyn_onoff_flag
		{
		}
	;

path
	: tokIDENTIFIER
		{
		}
	| path '.' tokIDENTIFIER
		{
		}
	;

group
	: group_start group_middle rwEND_GROUP
		{
		}
	;

group_start
	: rwGROUP tokIDENTIFIER
		{
		}
	;

group_middle
	:
		{
		}
	| group_middle node_flag
		{
		}
	| group_middle instance
		{
		}
	;

instance
	: instance_start instance_middle rwEND_INSTANCE
		{
		}
	;

instance_start
	: rwINSTANCE tokIDENTIFIER
		{
		}
	;

instance_middle
	:
		{
		}
	| instance_middle instance_flag
		{
		}
	| instance_middle transform
		{
		}
	;


instance_flag
	: rwID tokIDENTIFIER
		{
		}
	| node_flag
		{
		}
	;

transform
	: rwTRANSLATE dynvector
		{
		}
	| rwSCALE dynvector
		{
		}
	| rwROTATE dynvector '(' dynvalue ')'
		{
		}
	| lookat
		{
		}
	;

lookat
	: lookat_start lookat_middle rwEND_LOOK_AT
		{
		}
	;

lookat_start
	: rwLOOK_AT
		{
		}
	;

lookat_middle
	:
		{
		}
	| lookat_middle rwEYE dynvector
		{
		}
	| lookat_middle rwTARGET dynvector
		{
		}
	| lookat_middle rwUP dynvector
		{
		}
	;

object
	: object_start object_middle rwEND_OBJECT
		{
		  slide_gpCSLIDEParser->EndObject();
		}
	;

object_start
	: rwOBJECT tokIDENTIFIER id_list
		{ 
		  slide_gpCSLIDEParser->AddObject(@1.text, @1.first_line,
                                                  @1.first_column,
		                                  $2, $3);
		}
	;

object_middle
	:
		{
		}
	| object_middle rwSOLID dyn_solid_flag
		{
		}
	| object_middle node_flag
		{
		}
	;

face 	
	: face_start face_middle rwEND_FACE
		{ 
		  slide_gpCSLIDEParser->EndFace();
		}
	;

face_start 	
	: rwFACE tokIDENTIFIER id_list
		{ 
		  slide_gpCSLIDEParser->AddFace(@1.text, @1.first_line, 
                                                @1.first_column,
	                                        $2, $3);
		}
	;

face_middle
	:
		{
		}
	| face_middle rwSURFACE surface_id
		{
		}
	;

point	
	: point_start point_middle rwEND_POINT
		{ 
		  slide_gpCSLIDEParser->EndPoint();
		}
	;

point_start	
	: rwPOINT tokIDENTIFIER '(' value value value ')'
		{ 
		  Point pt;

		  pt.Set($4, $5, $6);
		  slide_gpCSLIDEParser->AddPoint(@1.text, @1.first_line, 
						 @1.first_column,
		                                 $2, pt);
		}
	;

point_middle	
	: 
		{
		}
	| point_middle rwSURFACE surface_id
		{
		}
	| point_middle rwNORMAL dynvector
		{ 
		}
	| point_middle rwTEXTURE dynpoint
		{ 
		}
	;

surface
	: surface_start surface_middle rwEND_SURFACE
		{
		}
	;

surface_start
	: rwSURFACE tokIDENTIFIER
		{
		}
	;

surface_middle
	: 
		{
		}
	| surface_middle rwCOLOR dynvector
		{
		}
	| surface_middle rwREFLECTIVITY dynvector
		{
		}
	| surface_middle rwEXPONENT dynvalue
		{
		}
	| surface_middle rwMETALLIC dynvalue
		{
		}
	;

id_list
	: '('
		{
		  $$ = slide_gpCSLIDEParser->CreateIDList();
		}
	| id_list tokIDENTIFIER
		{
		  $$ = slide_gpCSLIDEParser->InsertIDList($1, $2);
		}
	| id_list ')'
		{
		  $$ = $1;
		}
	;		

node_flag
	: rwSURFACE surface_id
		{
		}
	| rwSHADING dyn_shading_flag
		{
		}
	| rwLOD dyn_lod_flag
		{
		}
	;
	
surface_id
	: rwSLF_INHERIT 
		{
		  $$ = (CHAR*)(gcppcSLIDEReservedWords[sliderwtSLF_INHERIT]);
		}
	| tokIDENTIFIER
		{
		  $$ = $1;
		}
	;

dynpoint
	: '(' dynvalue dynvalue dynvalue ')'
		{
		  $$ = new DynPoint;
		  ASSERT( $$ != NULL );
		  $$->SetAll($2, $3, $4);
		  delete $2;
		  delete $3;
		  delete $4;
		}
	;

dynvector
	: '(' dynvalue dynvalue dynvalue ')'
		{
		  $$ = new DynVector;
		  ASSERT( $$ != NULL );
		  $$->SetAll($2, $3, $4);
		  delete $2;
		  delete $3;
		  delete $4;
		}
	;

dynvector2d
	: '(' dynvalue dynvalue ')'
		{
		  $$ = new DynVector;
		  ASSERT( $$ != NULL );
		  $$->SetAll($2, $3);
		  delete $2;
		  delete $3;
		}
	;

dynvalue	
	: value
		{ 
		  $$ = new DynFloat; 
		  ASSERT( $$ != NULL ); 
		  $$->SetValue($1);
		}
	| tokTCL_BLOCK
		{ 
		  $$ = new DynFloat; 
		  ASSERT( $$ != NULL ); 
		  $$->SetDynExpr($1);
		}
	;

value	
	: tokCONST_FLOAT
		{ $$ = $1; }
	| tokCONST_INT
		{ $$ = (FLOAT)$1; }
	;

dyn_onoff_flag
	: onoff_flag
		{ 
		  $$ = new DynInt; 
		  ASSERT( $$ != NULL ); 
		  $$->SetValue($1);
		}
	| tokTCL_BLOCK
		{ 
		  $$ = new DynInt; 
		  ASSERT( $$ != NULL ); 
		  $$->SetDynExpr($1);
		}
	;
	
onoff_flag
	: rwSLF_ON
		{ $$ = slideoftSLF_ON; }
	| rwSLF_OFF
		{ $$ = slideoftSLF_OFF; }
	;

dyn_solid_flag
	: solid_flag
		{
		  $$ = new DynInt;
		  ASSERT( $$ != NULL );
		  $$->SetValue($1);
		}
	| tokTCL_BLOCK
		{
		  $$ = new DynInt;
		  ASSERT( $$ != NULL );
		  $$->SetDynExpr($1);
		}
	;

solid_flag
	: rwSLF_SOLID
		{
		  $$ = slidestSLF_SOLID;
		}
	| rwSLF_HOLLOW
		{
		  $$ = slidestSLF_HOLLOW;
		}
	;

dyn_shading_flag
	: shading_flag
		{
		  $$ = new DynInt;
		  ASSERT( $$ != NULL );
		  $$->SetValue($1);
		}
	| tokTCL_BLOCK
		{
		  $$ = new DynInt;
		  ASSERT( $$ != NULL );
		  $$->SetDynExpr($1);
		}
	;

shading_flag
	: rwSLF_INHERIT
		{
		  $$ = slideshtSLF_INHERIT;
		}
	| rwSLF_WIRE
		{
		  $$ = slideshtSLF_WIRE;
		}
	| rwSLF_FLAT
		{
		  $$ = slideshtSLF_FLAT;
		}
	| rwSLF_GOURAUD
		{
		  $$ = slideshtSLF_GOURAUD;
		}
	| rwSLF_PHONG
		{
		  $$ = slideshtSLF_PHONG;
		}
	;

dyn_lod_flag
	: lod_flag
		{
		  $$ = new DynInt;
		  ASSERT( $$ != NULL );
		  $$->SetValue($1);
		}
	| tokTCL_BLOCK
		{
		  $$ = new DynInt;
		  ASSERT( $$ != NULL );
		  $$->SetDynExpr($1);
		}
	;

lod_flag
	: rwSLF_OFF
		{
		  $$ = slidelodtSLF_OFF;
		}
	| rwSLF_BOUND
		{
		  $$ = slidelodtSLF_BOUND;
		}
	| rwSLF_EDGES
		{
		  $$ = slidelodtSLF_EDGES;
		}
	| rwSLF_FULL
		{
		  $$ = slidelodtSLF_FULL;
		}
	;

dyn_projection_flag
	: projection_flag
		{
		  $$ = new DynInt;
		  ASSERT( $$ != NULL );
		  $$->SetValue($1);
		}
	| tokTCL_BLOCK
		{
		  $$ = new DynInt;
		  ASSERT( $$ != NULL );
		  $$->SetDynExpr($1);
		}
	;

projection_flag
	: rwSLF_PARALLEL
		{
		  $$ = slidepjtSLF_PARALLEL;
		}
	| rwSLF_PERSPECTIVE
		{
		  $$ = slidepjtSLF_PERSPECTIVE;
		}
	;

dyn_lighttype_flag
	: lighttype_flag
		{
		  $$ = new DynInt;
		  ASSERT( $$ != NULL );
		  $$->SetValue($1);
		}
	| tokTCL_BLOCK
		{
		  $$ = new DynInt;
		  ASSERT( $$ != NULL );
		  $$->SetDynExpr($1);
		}
	;

lighttype_flag
	: rwSLF_AMBIENT
		{
		  $$ = slideltSLF_AMBIENT;
		}
	| rwSLF_DIRECTIONAL
		{
		  $$ = slideltSLF_DIRECTIONAL;
		}
	| rwSLF_POINT
		{
		  $$ = slideltSLF_POINT;
		}
	| rwSLF_SPOT
		{
		  $$ = slideltSLF_SPOT;
		}
	;
%%

/*
	| rwGENERAL '(' value value value value value value value value value value value value value value value value ')'
		{
		  slide_gpCSLIDEParser->AddGeneral(@1.text, @1.first_line,
                                                   @1.first_column,
						   $3, $4, $5, $6,
						   $7, $8, $9, $10,
						   $11, $12, $13, $14,
						   $15, $16, $17, $18);
		}
*/

CSLIDEParser *slide_InitParser(CSLIDEParser *pCSLIDEParser, CHAR *pcInput, 
			       CHAR **ppcDirectories, UINT_32 uiDirectories)
{
  ASSERT( pcInput != NULL );

  ASSERT( pCSLIDEParser != NULL );

  pCSLIDEParser->Init(pcInput, ppcDirectories, uiDirectories, 
		      &slide_gpCSLIDEParser, 
		      &slide_lloc, yyparse,
		      &slide_gpCScanner,
		      &slide_leng,
		      slide_BeginInitial,
		      slide_GetCurrentBuffer,
		      slide_CreateBuffer,
		      slide_DeleteBuffer,
		      slide_SwitchToBuffer,
		      slide_SetValueInt,
		      slide_SetValueFloat,
		      slide_SetValueString);

  slide_gpCSLIDEParser = pCSLIDEParser;

  return pCSLIDEParser;
}

// This is yyerror which is called by bison
void slide_error(char * /*s*/)
{
  slide_gpCSLIDEParser->HandleError(yylloc.first_line, yylloc.first_column, "");
}
