#include "slideface.h"

#include "slideworld.h"
#include "slidepoint.h"

BOOL CSLIDEFace::m_bRenderStartPositionEnabled = FALSE;

//////////////////////////////////////////////////////////////////////////////
// CSLIDEFace Class
//

CSLIDEFace::CSLIDEFace()
{
  m_plistPointIDs = NULL;
}

CSLIDEFace::~CSLIDEFace()
{
}

VOID CSLIDEFace::Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, CHAR *pcID, 
		      CListInt *plistPointIDs)
{
  CSLIDEPrimitive::Init(pcFile, uiLine, uiColumn, pcID);

  ASSERT( plistPointIDs != NULL );

  m_plistPointIDs = plistPointIDs;
  m_listCSLIDEPoint.Init();
}

VOID CSLIDEFace::Uninit()
{
  CSLIDEPrimitive::Uninit();

  if ( m_plistPointIDs != NULL )
    {
      delete m_plistPointIDs;
      m_plistPointIDs = NULL;
    }
}

BOOL CSLIDEFace::Initialized()
{
  if ( !CSLIDEPrimitive::Initialized() )
    {
      return FALSE;
    }

  return ( m_plistPointIDs != NULL );
}

BOOL CSLIDEFace::CheckSemantics(CSLIDEWorld *pCSLIDEWorld)
{
  BOOL bCorrect;
  CListIterInt iter;
  CHAR *pcID;
  CSLIDEPoint *pCSLIDEPoint;

  if ( !CSLIDEPrimitive::CheckSemantics(pCSLIDEWorld) )
    {
      return FALSE;
    }

  bCorrect = TRUE;

  // Check Point References

  iter.Init(m_plistPointIDs);

  for ( pcID = (CHAR*)(iter.PeekFirst());
	pcID != NULL;
	pcID = (CHAR*)(iter.PeekNext()) )
    {
      pCSLIDEPoint = pCSLIDEWorld->PeekPoint(pcID);
      if ( pCSLIDEPoint == NULL )
	{
	  HandleError("Undefined %s \"%s\"", gcppcSLIDEReservedWords[sliderwtPoint], 
		      pcID);
	  bCorrect = FALSE;
	}
      m_listCSLIDEPoint.InsertLast(pCSLIDEPoint, 0);
    }

  return bCorrect;
}

BOOL CSLIDEFace::Valid()
{
  if ( !CSLIDEPrimitive::Valid() )
    {
      return FALSE;
    }

  return (GetPointCount() >= 3);
}

CSLIDEPrimitiveIter *CSLIDEFace::AllocateIter()
{
  CSLIDEFaceIter *pCSLIDEFaceIter;

  pCSLIDEFaceIter = new CSLIDEFaceIter;
  ASSERT(pCSLIDEFaceIter != NULL );
  pCSLIDEFaceIter->Init(this);

  return pCSLIDEFaceIter;
}

VOID CSLIDEFace::FreeIter(CSLIDEPrimitiveIter *pCSLIDEFaceIter)
{
  delete pCSLIDEFaceIter;
}

///////////////////////////////////////////////////
// START: ASSIGNMENT 2
// 

// Render the current position of the vertices, and conditionally render
// the starting positions of the vertices (if GetRenderStartPositionEnabled() 
// is TRUE).
//
// You may want to implement CSLIDEPoint::Render() and 
// CSLIDEPoint::RenderStartPosition()
VOID CSLIDEFace::Render()
{
  CListIterInt iter;
  CSLIDEPoint *pCSLIDEPoint;

  iter.Init(&m_listCSLIDEPoint);
	
  if ( GetRenderStartPositionEnabled() )
    {
      // Fix this
    }

  // Fix this
}

// This should return the first point in the face that is sufficiently
// close to the mouse cursor or NULL if there is no vertex near the mouse 
// cursor
CSLIDEPoint *CSLIDEFace::SelectPoint(FLOAT fX, FLOAT fY)
{
  CListIterInt iter;
  CSLIDEPoint *pCSLIDEPoint;

  iter.Init(&m_listCSLIDEPoint);

  // Fix this

  return NULL;
}

// Write the new SLIDE geometry description -- NO MORPHING
VOID CSLIDEFace::WriteSLIDE(FILE *fpOutputFile)
{
  CListIterInt iter;
  CSLIDEPoint *pCSLIDEPoint;

  ASSERT( fpOutputFile != NULL );

  fprintf(fpOutputFile, "%s %s ( ", gcppcSLIDEReservedWords[sliderwtFace], 
	  m_pcID);

  iter.Init(&m_listCSLIDEPoint);

  for ( pCSLIDEPoint = (CSLIDEPoint*)iter.PeekFirst(); 
	pCSLIDEPoint != NULL;
	pCSLIDEPoint = (CSLIDEPoint*)iter.PeekNext() )
    {
      fprintf(fpOutputFile, "%s ", pCSLIDEPoint->GetID());      
    }

  fprintf(fpOutputFile, ")\n");

  fprintf(fpOutputFile, "%s\n", 
	  gcppcSLIDEReservedWords[sliderwtEndFace]);
}

// Write the MORPHING SLIDE geometry description
VOID CSLIDEFace::WriteSLIDEMorph(FILE *fpOutputFile)
{
  // Fix this
}

// 
// END  : ASSIGNMENT 2
///////////////////////////////////////////////////

UINT_32 CSLIDEFace::GetPointCount()
{
  return m_listCSLIDEPoint.GetLength();
}

//////////////////////////////////////////////////////////////////////////////
// CSLIDEFaceIter Class
//

CSLIDEFaceIter::CSLIDEFaceIter()
{
}

CSLIDEFaceIter::~CSLIDEFaceIter()
{
}

VOID CSLIDEFaceIter::Init(CSLIDEFace *pCSLIDEFace)
{
  CSLIDEPrimitiveIter::Init(pCSLIDEFace);

  m_iter.Init(&(pCSLIDEFace->m_listCSLIDEPoint));
}

VOID CSLIDEFaceIter::Uninit()
{
  CSLIDEPrimitiveIter::Uninit();
}

CSLIDEPoint *CSLIDEFaceIter::PeekFirstCSLIDEPoint()
{
  return (CSLIDEPoint*)(m_iter.PeekFirst());
}

CSLIDEPoint *CSLIDEFaceIter::PeekNextCSLIDEPoint()
{
  return (CSLIDEPoint*)(m_iter.PeekNext());
}

CSLIDEPoint *CSLIDEFaceIter::PeekLastCSLIDEPoint()
{
  return (CSLIDEPoint*)(m_iter.PeekLast());
}

CSLIDEPoint *CSLIDEFaceIter::PeekPrevCSLIDEPoint()
{
  return (CSLIDEPoint*)(m_iter.PeekPrev());
}
