#include "slideparser.h"

#include "slideworld.h"
#include "window.h"
#include "slideobject.h"
#include "slideface.h"
#include "slidepoint.h"

CSLIDEParser::CSLIDEParser()
{
  m_pCSLIDEWorld = NULL;
  m_pCWindowCurr = NULL;
  m_pCSLIDEObjectCurr = NULL;
  m_pCSLIDEFaceCurr = NULL;
  m_pCSLIDEPointCurr = NULL;
}

CSLIDEParser::~CSLIDEParser()
{
}

VOID CSLIDEParser::Init(CHAR *pcFile, CHAR **ppcDirectories, UINT_32 uiDirectories, 
			CSLIDEParser **ppyy_CSLIDEParser, 
			YYLTYPE *pyylloc, YY_PARSE_TYPE *pfyy_Parse,
			CScanner **ppyy_CScanner,
			int *pyyleng,
			YY_BEGIN_INITIAL_TYPE *pfyy_BeginInitial,
			YY_GET_CURRENT_BUFFER_TYPE *pfyy_GetCurrentBuffer,
			YY_CREATE_BUFFER_TYPE *pfyy_CreateBuffer,
			YY_DELETE_BUFFER_TYPE *pfyy_DeleteBuffer,
			YY_SWITCH_TO_BUFFER_TYPE *pfyy_SwitchToBuffer,
			YY_SET_VALUE_INT_TYPE *pfyy_SetValueInt,
			YY_SET_VALUE_FLOAT_TYPE *pfyy_SetValueFloat,
			YY_SET_VALUE_STRING_TYPE *pfyy_SetValueString)
{
  CParser::Init(pcFile, ppcDirectories, uiDirectories, 
		(CParser**)ppyy_CSLIDEParser, 
		pyylloc, pfyy_Parse,
		ppyy_CScanner,
		pyyleng,
		pfyy_BeginInitial,
		pfyy_GetCurrentBuffer,
		pfyy_CreateBuffer,
		pfyy_DeleteBuffer,
		pfyy_SwitchToBuffer,
		pfyy_SetValueInt,
		pfyy_SetValueFloat,
		pfyy_SetValueString);

  m_pCSLIDEWorld = CreateWorld();
  ASSERT( m_pCSLIDEWorld != NULL );
  m_pCWindowCurr = NULL;
  m_pCSLIDEObjectCurr = NULL;
  m_pCSLIDEFaceCurr = NULL;
  m_pCSLIDEPointCurr = NULL;

  SetAST(m_pCSLIDEWorld);
}

VOID CSLIDEParser::Uninit()
{
  CParser::Uninit();
}

VOID CSLIDEParser::AddWindow(CHAR *pcFile, UINT_32 uiLine, 
			      UINT_32 uiColumn,
			      CHAR *pcID)
{
  CWindow *pCWindow;

  ASSERT( m_pCWindowCurr == NULL );

  pCWindow = m_pCSLIDEWorld->PeekWindow(pcID);
  if ( pCWindow != NULL )
    {
      HandleError(uiLine, uiColumn, "%s \"%s\" previously defined at %s: %ld: %ld", 
		  gcppcSLIDEReservedWords[sliderwtWindow],
		  pcID, pCWindow->GetFileName(), 
		  pCWindow->GetFileLine(), 
		  pCWindow->GetFileColumn());
    }

  pCWindow = CreateWindow(pcFile, uiLine, uiColumn, pcID);
  ASSERT( pCWindow != NULL );

  VERIFY(m_pCSLIDEWorld->InsertWindow(pCWindow));
  m_pCWindowCurr = pCWindow;
}

VOID CSLIDEParser::EndWindow()
{
  ASSERT( m_pCWindowCurr != NULL );
  m_pCWindowCurr = NULL;
}

VOID CSLIDEParser::SetWindowOrigin(FLOAT fX, FLOAT fY)
{
  ASSERT( m_pCWindowCurr != NULL );

  m_pCWindowCurr->SetOrigin(fX, fY);
}

VOID CSLIDEParser::SetWindowSize(FLOAT fX, FLOAT fY)
{
  ASSERT( m_pCWindowCurr != NULL );

  m_pCWindowCurr->SetSize(fX, fY);
}

VOID CSLIDEParser::SetWindowBackground(DynVector *pdynv)
{
  ASSERT( m_pCWindowCurr != NULL );

   m_pCWindowCurr->SetBackground(pdynv);
}

VOID CSLIDEParser::AddObject(CHAR *pcFile, UINT_32 uiLine, 
			     UINT_32 uiColumn,
			     CHAR *pcID, 
			     CListInt *plistFaceIDs)
{
  CSLIDEObject *pCSLIDEObject;

  ASSERT( m_pCSLIDEObjectCurr == NULL );

  pCSLIDEObject = m_pCSLIDEWorld->PeekObject(pcID);
  if ( pCSLIDEObject != NULL )
    {
      HandleError(uiLine, uiColumn, "%s \"%s\" previously defined at %s: %ld: %ld", 
		  gcppcSLIDEReservedWords[sliderwtObject],
		  pcID, pCSLIDEObject->GetFileName(), 
		  pCSLIDEObject->GetFileLine(), 
		  pCSLIDEObject->GetFileColumn());
    }

  pCSLIDEObject = CreateObject(pcFile, uiLine, uiColumn,
			     pcID, plistFaceIDs);
  ASSERT( pCSLIDEObject != NULL );

  VERIFY(m_pCSLIDEWorld->InsertObject(pCSLIDEObject));
  m_pCSLIDEObjectCurr = pCSLIDEObject;
}

VOID CSLIDEParser::EndObject()
{
  ASSERT( m_pCSLIDEObjectCurr != NULL );
  m_pCSLIDEObjectCurr = NULL;
}

VOID CSLIDEParser::AddFace(CHAR *pcFile, UINT_32 uiLine, 
			   UINT_32 uiColumn,
			   CHAR *pcID, 
			   CListInt *plistPointIDs)
{
  CSLIDEFace *pCSLIDEFace;

  ASSERT( m_pCSLIDEFaceCurr == NULL );

  pCSLIDEFace = m_pCSLIDEWorld->PeekFace(pcID);
  if ( pCSLIDEFace != NULL )
    {
      HandleError(uiLine, uiColumn, "%s \"%s\" previously defined at %s: %ld: %ld",  
		  gcppcSLIDEReservedWords[sliderwtFace],
		  pcID, pCSLIDEFace->GetFileName(), 
		  pCSLIDEFace->GetFileLine(), 
		  pCSLIDEFace->GetFileColumn());
    }

  pCSLIDEFace = CreateFace(pcFile, uiLine, uiColumn, 
			   pcID, plistPointIDs);
  ASSERT( pCSLIDEFace != NULL );

  VERIFY(m_pCSLIDEWorld->InsertFace(pCSLIDEFace));
  m_pCSLIDEFaceCurr = pCSLIDEFace;
}

VOID CSLIDEParser::EndFace()
{
  ASSERT( m_pCSLIDEFaceCurr != NULL );
  m_pCSLIDEFaceCurr = NULL;
}

VOID CSLIDEParser::AddPoint(CHAR *pcFile, UINT_32 uiLine, 
			    UINT_32 uiColumn, CHAR *pcID,
			    const Point &pt)
{
  CSLIDEPoint *pCSLIDEPoint;

  ASSERT( m_pCSLIDEPointCurr == NULL );

  pCSLIDEPoint = m_pCSLIDEWorld->PeekPoint(pcID);
  if ( pCSLIDEPoint != NULL )
    {
      HandleError(uiLine, uiColumn, "%s \"%s\" previously defined at %s: %ld: %ld", 
		  gcppcSLIDEReservedWords[sliderwtPoint],
		  pcID, pCSLIDEPoint->GetFileName(), 
		  pCSLIDEPoint->GetFileLine(), 
		  pCSLIDEPoint->GetFileColumn());
    }

  pCSLIDEPoint = CreatePoint(pcFile, uiLine, uiColumn,
			     pcID, pt);
  ASSERT( pCSLIDEPoint != NULL );

  VERIFY(m_pCSLIDEWorld->InsertPoint(pCSLIDEPoint));
  m_pCSLIDEPointCurr = pCSLIDEPoint;
}

VOID CSLIDEParser::EndPoint()
{
  ASSERT( m_pCSLIDEPointCurr != NULL );
  m_pCSLIDEPointCurr = NULL;
}

CListInt *CSLIDEParser::CreateIDList()
{
  return CreateList();
}

CListInt *CSLIDEParser::InsertIDList(CListInt *pCListInt, CHAR *pcID)
{
  return InsertList(pCListInt, pcID);
}

CSLIDEWorld *CSLIDEParser::CreateWorld()
{
  CSLIDEWorld *pCSLIDEWorld;

  pCSLIDEWorld = new CSLIDEWorld();
  ASSERT( pCSLIDEWorld != NULL );
  pCSLIDEWorld->Init();

  return pCSLIDEWorld;
}

CWindow *CSLIDEParser::CreateWindow(CHAR *pcFile, UINT_32 uiLine, 
				    UINT_32 uiColumn,
				    CHAR *pcID)
{
  CWindow *pCWindow;

  pCWindow = new CWindow();
  ASSERT( pCWindow != NULL );
  pCWindow->Init(pcFile, uiLine, uiColumn, pcID);

  return pCWindow;
}

CSLIDEObject *CSLIDEParser::CreateObject(CHAR *pcFile, UINT_32 uiLine, 
					 UINT_32 uiColumn,
					 CHAR *pcID, 
					 CListInt *plistFaceIDs)
{
  CSLIDEObject *pCSLIDEObject;

  pCSLIDEObject = new CSLIDEObject();
  ASSERT( pCSLIDEObject != NULL );
  pCSLIDEObject->Init(pcFile, uiLine, uiColumn, pcID,
		      plistFaceIDs);

  return pCSLIDEObject;
}

CSLIDEFace *CSLIDEParser::CreateFace(CHAR *pcFile, UINT_32 uiLine, 
				     UINT_32 uiColumn,
				     CHAR *pcID, 
				     CListInt *plistPointIDs)
{
  CSLIDEFace *pCSLIDEFace;

  pCSLIDEFace = new CSLIDEFace();
  ASSERT( pCSLIDEFace != NULL );
  pCSLIDEFace->Init(pcFile, uiLine, uiColumn, pcID, plistPointIDs);

  return pCSLIDEFace;
}

CSLIDEPoint *CSLIDEParser::CreatePoint(CHAR *pcFile, UINT_32 uiLine, 
				       UINT_32 uiColumn,
				       CHAR *pcID, 
				       const Point &pt)
{
  CSLIDEPoint *pCSLIDEPoint;

  pCSLIDEPoint = new CSLIDEPoint();
  ASSERT( pCSLIDEPoint != NULL );
  pCSLIDEPoint->Init(pcFile, uiLine, uiColumn, pcID, pt);

  return pCSLIDEPoint;
}

CListInt *CSLIDEParser::CreateList()
{
  CListInt *pCListInt;

  pCListInt = new CListInt();
  ASSERT( pCListInt != NULL );

  pCListInt->Init();

  return pCListInt;
}

CListInt *CSLIDEParser::InsertList(CListInt *pCListInt, VOID *pvData)
{
  ASSERT( pCListInt != NULL );

  pCListInt->InsertLast(pvData, 0);

  return pCListInt;
}


