#ifndef __SLIDEPARSER_H__
#define __SLIDEPARSER_H__

#include "parser.h"
#include "dynamics.h"
#include "point.h"
#include "vector.h"

#include "slide.h"
class CSLIDEWorld;
class CWindow;
class CSLIDEObject;
class CSLIDEFace;
class CSLIDEPoint;

//////////////////////////////////////////////////////////////////////////////
// CSLIDEParser Class
//

class CSLIDEParser : public CParser
{
//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEParser();
  virtual ~CSLIDEParser();
  virtual VOID Init(CHAR *pcFile, CHAR **ppcDirectories, UINT_32 uiDirectories, 
		    CSLIDEParser **ppyy_CSLIDEParser, 
		    YYLTYPE *pyylloc, YY_PARSE_TYPE *pfyy_Parse,
		    CScanner **ppyy_CScanner,
		    int *pyyleng,
		    YY_BEGIN_INITIAL_TYPE *pfyy_BeginInitial,
		    YY_GET_CURRENT_BUFFER_TYPE *pfyy_GetCurrentBuffer,
		    YY_CREATE_BUFFER_TYPE *pfyy_CreateBuffer,
		    YY_DELETE_BUFFER_TYPE *pfyy_DeleteBuffer,
		    YY_SWITCH_TO_BUFFER_TYPE *pfyy_SwitchToBuffer,
		    YY_SET_VALUE_INT_TYPE *pfyy_SetValueInt,
		    YY_SET_VALUE_FLOAT_TYPE *pfyy_SetValueFloat,
		    YY_SET_VALUE_STRING_TYPE *pfyy_SetValueString);
  virtual VOID Uninit();

//////////////////////////////////////////////////
// AST Functions
//
public:
  virtual VOID AddWindow(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			  CHAR *pcID);
  virtual VOID EndWindow();
  virtual VOID SetWindowOrigin(FLOAT fX, FLOAT fY);
  virtual VOID SetWindowSize(FLOAT fX, FLOAT fY);
  virtual VOID SetWindowBackground(DynVector *pdynv);
  virtual VOID AddObject(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			 CHAR *pcID, 
			 CListInt *plistFaceIDs);
  virtual VOID EndObject();
  virtual VOID AddFace(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
		       CHAR *pcID, CListInt *plistPointIDs);
  virtual VOID EndFace();
  virtual VOID AddPoint(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			CHAR *pcID, const Point &pt);
  virtual VOID EndPoint();
  virtual CListInt *CreateIDList();
  virtual CListInt *InsertIDList(CListInt *pCListInt, CHAR *pcID);

protected:
  virtual CSLIDEWorld *CreateWorld();

// Cheap WIN32 hack
#ifdef CreateWindowA
#undef CreateWindowA
#endif


  virtual CWindow *CreateWindow(CHAR *pcFile, UINT_32 uiLine, 
				UINT_32 uiColumn,
				CHAR *pcID);
  virtual CSLIDEObject *CreateObject(CHAR *pcFile, UINT_32 uiLine, 
				     UINT_32 uiColumn,
				     CHAR *pcID, 
				     CListInt *plistFaceIDs);
  virtual CSLIDEFace *CreateFace(CHAR *pcFile, UINT_32 uiLine, 
				 UINT_32 uiColumn,
				 CHAR *pcID, 
				 CListInt *plistPointIDs);
  virtual CSLIDEPoint *CreatePoint(CHAR *pcFile, UINT_32 uiLine, 
				   UINT_32 uiColumn,
				   CHAR *pcID, 
				   const Point &pt);
  CListInt *CreateList();
  CListInt *InsertList(CListInt *pCListInt, VOID *pvData);

//////////////////////////////////////////////////
// Accessor Functions
//
public:

//////////////////////////////////////////////////
// Class Variables
//

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CSLIDEWorld *m_pCSLIDEWorld;
  CWindow *m_pCWindowCurr;
  CSLIDEObject *m_pCSLIDEObjectCurr;
  CSLIDEFace *m_pCSLIDEFaceCurr;
  CSLIDEPoint *m_pCSLIDEPointCurr;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

//////////////////////////////////////////////////////////////////////
// Globals
//

extern CSLIDEParser *slide_InitParser(CSLIDEParser *pCSLIDEParser, 
				      CHAR *pcInput, 
				      CHAR **ppcDirectories, 
				      UINT_32 uiDirectories);

#endif // __SLIDEPARSER_H__
