#ifndef __WINDOW_H__
#define __WINDOW_H__

#include "genlib.h"
#include "mathlib.h"
#include "gillib.h"
#include "togl.h"
#include "ootlib.h"

class CSLIDEParse;
class CSLIDEWorld;
class CSLIDEObject;
class CSLIDEPoint;

//////////////////////////////////////////////////////////////////////
// Class CWindow
//

const int cuiCWindowOotMethods = 7;

#define CWINDOW_METHODS \
  {"handleKeyboard", OOT_PARAM_TYPE_KEYBOARD, CWindow::CBHandleKeyboard}, \
  {"handleMouseButton", OOT_PARAM_TYPE_INT_4, CWindow::CBHandleMouseButton}, \
  {"handleMouseMotion", OOT_PARAM_TYPE_INT_3, CWindow::CBHandleMouseMotion}, \
  {"handleWrite", OOT_PARAM_TYPE_VOID, CWindow::CBHandleWrite}, \
  {"handleWriteMorph", OOT_PARAM_TYPE_VOID, CWindow::CBHandleWriteMorph}, \
  {"setDrawStartPosition", OOT_PARAM_TYPE_BOOL, CWindow::CBSetDrawStartPosition}, \
  {"setDisplayEvents", OOT_PARAM_TYPE_BOOL, CWindow::CBSetDisplayEvents}

#define CWINDOW_FIELDS_ENUMS \
  ftFramesPerSecondRequested, \
  ftUpdateTime, \
  ftRenderTime, \
  ftDrawStartPosition, \
  ftDisplayEvents

#define CWINDOW_FIELDS_STRINGS \
  "framesPerSecondRequested", \
  "updateTime", \
  "renderTime", \
  "drawStartPosition", \
  "displayEvents"

class CWindow
{
//////////////////////////////////////////////////
// Enumerated Types
//
public:
  enum EventLevelType 
    {
      eltNone,
      eltAction,
      eltCallBack,
      eltSize
    };

//////////////////////////////////////////////////
// Initialization
//
public:
  CWindow();
  virtual ~CWindow();
  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
		    CHAR *pcSLIDEID);
  virtual VOID Uninit();

  virtual VOID InitTcl();
  virtual VOID InitTogl();

  virtual VOID HandleDestroy();

//////////////////////////////////////////////////
// SLIDE Stuff
//
public:
  virtual BOOL CreateTclToplevel();
  virtual BOOL CheckSemantics(CSLIDEWorld *pCSLIDEWorld);
  virtual VOID Preprocess();

//////////////////////////////////////////////////
// Handlers
//
public:
  virtual VOID HandleReshape(INT_32 iWidth, INT_32 iHeight);
  virtual VOID HandleDisplay();
////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 2
// 
  virtual VOID HandleMouseButton(INT_32 iButton, INT_32 iState, 
				 INT_32 iX, INT_32 iY);
  virtual VOID HandleMouseMotion(INT_32 iX, INT_32 iY);
// 
// END  : ASSIGNMENT 2
/////////////////////////////////////////////////////////////////////////
  virtual VOID HandleKeyboard(INT_32 iKey, INT_32 iState, 
			      INT_32 iX, INT_32 iY);


  virtual VOID HandleWrite();
  virtual VOID HandleWriteMorph();

  virtual VOID PostRedisplay();

//////////////////////////////////////////////////
// Utilities
//
public:

protected:
  VOID DisplayEvent(EventLevelType eEventLevel, CHAR *pcFormat, ...);

public:
  inline VOID WinToVP(INT_32 &iXVP, INT_32 &iYVP, INT_32 iXWin, INT_32 iYWin); 
  inline VOID VPToNorm(FLOAT &fXNorm, FLOAT &fYNorm, INT_32 iXVP, INT_32 iYVP);

//////////////////////////////////////////////////
// CallBack Functions / Handler Dispatchers
//
public:
  static int CreateOotClass(Tcl_Interp *pInterp);
  static int CBHandleCreate(Tcl_Interp *pInterp, OotClass *pClass, 
			    int argc, char **argv);
  static VOID CBHandleDestroy(ClientData pvClientData);

  static VOID CBHandleToglCreate(struct Togl *pTogl);
  static VOID CBHandleDisplay(struct Togl *pTogl);
  static VOID CBHandleReshape(struct Togl *pTogl);
  static VOID CBHandleDestroy(struct Togl *pTogl);

  static int CBHandleMouseButton(Tcl_Interp *pInterp, OotObject *pObject,
				 INT_32 iButton, INT_32 iState,
				 INT_32 iX, INT_32 iY);
  static int CBHandleMouseMotion(Tcl_Interp *pInterp, OotObject *pObject,
				 INT_32 iButton, INT_32 iX, INT_32 iY);
  static int CBHandleKeyboard(Tcl_Interp *pInterp, OotObject *pObject,
			      CHAR *pcKeyCode, INT_32 iState, 
			      INT_32 iX, INT_32 iY);

  static int CBHandleWrite(Tcl_Interp *pInterp, OotObject *pObject);
  static int CBHandleWriteMorph(Tcl_Interp *pInterp, OotObject *pObject);

  static int CBSetDrawStartPosition(Tcl_Interp *pInterp, OotObject *pObject, 
				    BOOL b);
  static int CBSetDisplayEvents(Tcl_Interp *pInterp, OotObject *pObject, 
				BOOL b);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  inline VOID SetTogl(struct Togl *pTogl);

  inline CHAR *GetFileName();
  inline UINT_32 GetFileLine();
  inline UINT_32 GetFileColumn();
  inline CHAR *GetSLIDEID();
  inline VOID SetOrigin(FLOAT fX, FLOAT fY);
  inline VOID SetSize(FLOAT fX, FLOAT fY);
  inline VOID SetBackground(DynVector *pdynvColor);

  // UI Stuff
  VOID SetDrawStartPosition(BOOL b);
  VOID SetDisplayEvents(BOOL b);

  VOID SetSecondsPerFrameRequested(DOUBLE dSecondsPerFrame);
  VOID SetUpdateTime(DOUBLE dUpdateTime);
  VOID SetRenderTime(DOUBLE dRenderTime);

//////////////////////////////////////////////////
// Class Accessor Functions
//
public:
  static inline VOID SetUIConfigFile(CHAR *pcUIConfigFile);
  static inline const CHAR *GetUIConfigFile();

//////////////////////////////////////////////////
// Class Variables
//
protected:
  // Table for decoding Tcl key names in Keyboard Callback
  static CHashString *m_pHashKeyboard;

  // OOT registration variables
  enum FieldType 
    {
      CWINDOW_FIELDS_ENUMS,

      ftSize
    };

  static Tcl_Interp *m_pInterp;
  static const int m_cuiOotMethods;
  static OotMethodEntry m_pOotMethods[cuiCWindowOotMethods];
  static const CHAR *m_cppcOotFields[ftSize];
  static const CHAR *m_cpcOotClassName;
  static OotClass *m_pOotClass;

  // Object Counter Variable
  static UINT_32 m_uiOotCounter;

  static CHAR *m_pcUIConfigFile;

//////////////////////////////////////////////////
// Member Variables
//
protected:
  // Tcl and Togl variables for Windowing
  UINT_32 m_uiOotID;
  CHAR *m_pcOotName;
  OotObject *m_pOotObject;
  struct Togl *m_pTogl;
  BOOL m_bRedisplayPending;

  INT_32 m_iWidth;
  INT_32 m_iHeight;

  // For Debugging Print Statements
  BOOL m_bDisplayEvents;        
  EventLevelType m_eEventLevel;

  // The SLIDE World which we are rendering
  CHAR *m_pcFile;
  UINT_32 m_uiLine;
  UINT_32 m_uiColumn;
  CHAR *m_pcSLIDEID;
  Vector m_vOrigin;
  Vector m_vSize;
  DynVector m_vBackground;
  CSLIDEWorld *m_pCSLIDEWorld;
  CSLIDEObject *m_pCSLIDEObject;

  // Output file
  CHAR *m_pcOutputFile;
  FILE *m_fpOutputFile;

  // 2D Editor UI Variables
  BOOL m_bLeftMouseDown;
  BOOL m_bMiddleMouseDown;
  BOOL m_bRightMouseDown;
  CSLIDEPoint *m_pCSLIDEPointSelected;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline VOID CWindow::SetUIConfigFile(CHAR *pcUIConfigFile)
{
  if ( m_pcUIConfigFile != NULL )
    {
      delete []m_pcUIConfigFile;
      m_pcUIConfigFile = NULL;
    }
  m_pcUIConfigFile = new CHAR[strlen(pcUIConfigFile)+1];
  ASSERT(m_pcUIConfigFile != NULL );
  strcpy(m_pcUIConfigFile, pcUIConfigFile);
}

inline const CHAR *CWindow::GetUIConfigFile()
{
  return m_pcUIConfigFile;
}

inline VOID CWindow::WinToVP(INT_32 &iXVP, INT_32 &iYVP, INT_32 iXWin, INT_32 iYWin)
{
  iXVP = iXWin;
  iYVP = m_iHeight - iYWin;
}

inline VOID CWindow::VPToNorm(FLOAT &fXNorm, FLOAT &fYNorm, 
                              INT_32 iXVP, INT_32 iYVP)
{
  fXNorm = (FLOAT)iXVP / (FLOAT)(m_iWidth);
  fYNorm = (FLOAT)iYVP / (FLOAT)(m_iHeight);
}

inline VOID CWindow::SetTogl(struct Togl *pTogl)
{
  ASSERT( pTogl != NULL );
  m_pTogl = pTogl;
}

inline CHAR *CWindow::GetFileName()
{
  return m_pcFile;
}

inline UINT_32 CWindow::GetFileLine()
{
  return m_uiLine;
}

inline UINT_32 CWindow::GetFileColumn()
{
  return m_uiColumn;
}

inline CHAR *CWindow::GetSLIDEID()
{
  return m_pcSLIDEID;
}

inline VOID CWindow::SetOrigin(FLOAT fX, FLOAT fY)
{
  m_vOrigin.Set(fX, fY, 0.0);
}

inline VOID CWindow::SetSize(FLOAT fX, FLOAT fY)
{
  m_vSize.Set(fX, fY, 0.0);
}

inline VOID CWindow::SetBackground(DynVector *pdynvColor)
{
  m_vBackground.SetAll(pdynvColor);
}

//////////////////////////////////////////////////////////////////////
// Globals
//

#endif __WINDOW_H__




